@@ -19,7 +19,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2008 Red Hat, Inc.
+ * (C) Copyright 2007 - 2010 Red Hat, Inc.
  * (C) Copyright 2007 - 2008 Novell, Inc.
  */
 
@@ -30,24 +30,12 @@
 #include <unistd.h>
 
 #include <gtk/gtk.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtklabel.h>
 #include <gdk/gdkx.h>
 #include <glib/gi18n.h>
 
-#ifdef NO_POLKIT_GNOME
-#include "polkit-gnome.h"
-#else
-#include <polkit-gnome/polkit-gnome.h>
-#endif
-
 #include <nm-setting-connection.h>
 #include <nm-setting-ip4-config.h>
+#include <nm-setting-ip6-config.h>
 #include <nm-setting-wired.h>
 #include <nm-setting-8021x.h>
 #include <nm-setting-wireless.h>
@@ -69,11 +57,12 @@
 #include "page-wireless.h"
 #include "page-wireless-security.h"
 #include "page-ip4.h"
+#include "page-ip6.h"
 #include "page-dsl.h"
 #include "page-mobile.h"
 #include "page-ppp.h"
 #include "page-vpn.h"
-#include "polkit-helpers.h"
+#include "ce-polkit-button.h"
 
 G_DEFINE_TYPE (NMConnectionEditor, nm_connection_editor, G_TYPE_OBJECT)
 
@@ -88,6 +77,13 @@ static gboolean nm_connection_editor_set_connection (NMConnectionEditor *editor,
                                                      NMConnection *connection,
                                                      GError **error);
 
+struct GetSecretsInfo {
+	NMConnectionEditor *self;
+	CEPage *page;
+	char *setting_name;
+	gboolean canceled;
+};
+
 static void
 nm_connection_editor_update_title (NMConnectionEditor *editor)
 {
@@ -140,13 +136,83 @@ ui_to_setting (NMConnectionEditor *editor)
 	return TRUE;
 }
 
+static gboolean
+editor_is_initialized (NMConnectionEditor *editor)
+{
+	return (g_slist_length (editor->initializing_pages) == 0);
+}
+
+static void
+update_sensitivity (NMConnectionEditor *editor)
+{
+	NMSettingConnection *s_con;
+	gboolean actionable = FALSE, authorized = FALSE, sensitive = FALSE;
+	GtkWidget *widget;
+	GSList *iter;
+
+	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (editor->connection, NM_TYPE_SETTING_CONNECTION));
+
+	/* Can't modify read-only connections; can't modify anything before the
+	 * editor is initialized either.
+	 */
+	if (   !nm_setting_connection_get_read_only (s_con)
+	    && editor_is_initialized (editor)) {
+		if (editor->system_settings_can_modify) {
+			actionable = ce_polkit_button_get_actionable (CE_POLKIT_BUTTON (editor->ok_button));
+			authorized = ce_polkit_button_get_authorized (CE_POLKIT_BUTTON (editor->ok_button));
+		}
+
+		if (editor->orig_scope == NM_CONNECTION_SCOPE_SYSTEM) {
+			/* If the user cannot ever be authorized to change system connections, and
+			 * the connection is a system connection, we desensitize the entire dialog.
+			 */
+			sensitive = authorized;
+		} else {
+			/* Otherwise, if the connection is originally a user-connection,
+			 * then everything is sensitive except possible the system checkbutton,
+			 * which will be insensitive if the user has no possibility of
+			 * authorizing.
+			 */
+			sensitive = TRUE;
+		}
+	}
+
+	gtk_widget_set_sensitive (GTK_WIDGET (editor->system_checkbutton), actionable && authorized);
+
+	/* Cancel button is always sensitive */
+	gtk_widget_set_sensitive (GTK_WIDGET (editor->cancel_button), TRUE);
+
+	widget = glade_xml_get_widget (editor->xml, "connection_name_label");
+	gtk_widget_set_sensitive (widget, sensitive);
+
+	widget = glade_xml_get_widget (editor->xml, "connection_name");
+	gtk_widget_set_sensitive (widget, sensitive);
+
+	widget = glade_xml_get_widget (editor->xml, "connection_autoconnect");
+	gtk_widget_set_sensitive (widget, sensitive);
+
+	widget = glade_xml_get_widget (editor->xml, "connection_name");
+	gtk_widget_set_sensitive (widget, sensitive);
+
+	for (iter = editor->pages; iter; iter = g_slist_next (iter)) {
+		widget = ce_page_get_page (CE_PAGE (iter->data));
+		gtk_widget_set_sensitive (widget, sensitive);
+	}
+}
+
 static void
 connection_editor_validate (NMConnectionEditor *editor)
 {
+	NMSettingConnection *s_con;
 	gboolean valid = FALSE;
 	GSList *iter;
 
-	if (!editor->initialized)
+	if (!editor_is_initialized (editor))
+		goto done;
+
+	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (editor->connection, NM_TYPE_SETTING_CONNECTION));
+	g_assert (s_con);
+	if (nm_setting_connection_get_read_only (s_con))
 		goto done;
 
 	if (!ui_to_setting (editor))
@@ -169,122 +235,47 @@ connection_editor_validate (NMConnectionEditor *editor)
 	valid = TRUE;
 
 done:
-	g_object_set (editor->system_gnome_action, "master-sensitive", valid, NULL);
+	ce_polkit_button_set_master_sensitive (CE_POLKIT_BUTTON (editor->ok_button), valid);
+	update_sensitivity (editor);
+}
+
+static void
+ok_button_actionable_cb (GtkWidget *button,
+                         gboolean actionable,
+                         NMConnectionEditor *editor)
+{
+	connection_editor_validate (editor);
+}
+
+static void
+can_modify_changed_cb (NMRemoteSettingsSystem *settings,
+                       GParamSpec *pspec,
+                       NMConnectionEditor *editor)
+{
+	connection_editor_validate (editor);
 }
 
 static void
 system_checkbutton_toggled_cb (GtkWidget *widget, NMConnectionEditor *editor)
 {
-	gboolean req_privs = FALSE;
-	NMSettingConnection *s_con;
+	gboolean use_polkit = TRUE;
 
-	/* If the connection was originally a system connection, obviously
-	 * privileges are required to change it.  If it was originally a user
-	 * connection, but the user requests that it be changed to a system
-	 * connection, privileges are also required.
+	/* The only time the Apply button does not need to use polkit is when the
+	 * original connection scope was USER and the "system" checkbutton is
+	 * unchecked.
 	 */
+	if (   !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))
+	    && (editor->orig_scope == NM_CONNECTION_SCOPE_USER))
+		use_polkit = FALSE;
 
-	if (editor->orig_scope == NM_CONNECTION_SCOPE_USER) {
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
-			req_privs = TRUE;
-	} else
-		req_privs = TRUE;
-
-	if (req_privs)
-		g_object_set (editor->system_gnome_action, "polkit-action", editor->system_action, NULL);
-	else
-		g_object_set (editor->system_gnome_action, "polkit-action", NULL, NULL);
-
-	/* Can't ever modify read-only connections */
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (editor->connection, NM_TYPE_SETTING_CONNECTION));
-	if (nm_setting_connection_get_read_only (s_con))
-		gtk_widget_set_sensitive (editor->ok_button, FALSE);
-
-	if (editor->initialized)
-		connection_editor_validate (editor);
-}
-
-static void
-set_editor_sensitivity (NMConnectionEditor *editor, gboolean sensitive)
-{
-	GtkWidget *widget;
-	GSList *iter;
-
-	/* Cancel button is always sensitive */
-	gtk_widget_set_sensitive (GTK_WIDGET (editor->cancel_button), TRUE);
-
-	widget = glade_xml_get_widget (editor->xml, "connection_name_label");
-	gtk_widget_set_sensitive (widget, sensitive);
-
-	widget = glade_xml_get_widget (editor->xml, "connection_name");
-	gtk_widget_set_sensitive (widget, sensitive);
-
-	widget = glade_xml_get_widget (editor->xml, "connection_autoconnect");
-	gtk_widget_set_sensitive (widget, sensitive);
-
-	widget = glade_xml_get_widget (editor->xml, "connection_name");
-	gtk_widget_set_sensitive (widget, sensitive);
-
-	if (editor->system_settings_can_modify)
-		gtk_widget_set_sensitive (GTK_WIDGET (editor->system_checkbutton), sensitive);
-
-	for (iter = editor->pages; iter; iter = g_slist_next (iter)) {
-		widget = ce_page_get_page (CE_PAGE (iter->data));
-		gtk_widget_set_sensitive (widget, sensitive);
-	}
-}
-
-static void
-update_sensitivity (NMConnectionEditor *editor, PolKitResult pk_result)
-{
-	NMSettingConnection *s_con;
-	gboolean denied = FALSE;
-
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (editor->connection, NM_TYPE_SETTING_CONNECTION));
-	/* Can't ever modify read-only connections */
-	if (nm_setting_connection_get_read_only (s_con)) {
-		set_editor_sensitivity (editor, FALSE);
-		return;
-	}
-
-	if (pk_result == POLKIT_RESULT_UNKNOWN)
-		pk_result = polkit_gnome_action_get_polkit_result (editor->system_gnome_action);
-
-	if (pk_result == POLKIT_RESULT_NO || pk_result == POLKIT_RESULT_UNKNOWN)
-		denied = TRUE;
-
-	switch (editor->orig_scope) {
-	case NM_CONNECTION_SCOPE_SYSTEM:
-		/* If the user cannot ever be authorized to change system connections, and
-		 * the connection is a system connection, we desensitize the entire dialog.
-		 */
-		set_editor_sensitivity (editor, !denied);
-		break;
-	default:
-		/* If the user cannot ever be authorized to change system connections, and
-		 * the connection is a user connection, we desensitize system_checkbutton.
-		 */
-		set_editor_sensitivity (editor, TRUE);
-		if (denied)
-			gtk_widget_set_sensitive (GTK_WIDGET (editor->system_checkbutton), FALSE);
-		break;
-	}
-}
-
-static void
-system_pk_result_changed_cb (PolKitGnomeAction *gnome_action,
-                             PolKitResult result,
-                             NMConnectionEditor *editor)
-{
-	update_sensitivity (editor, result);
+	ce_polkit_button_set_use_polkit (CE_POLKIT_BUTTON (editor->ok_button), use_polkit);
+	connection_editor_validate (editor);
 }
 
 static void
 nm_connection_editor_init (NMConnectionEditor *editor)
 {
-	GtkWidget *dialog, *hbox;
-	const char *auth_label, *auth_tooltip;
-	const char *label, *tooltip;
+	GtkWidget *dialog;
 
 	/* Yes, we mean applet.glade, not nm-connection-editor.glade. The wireless security bits
 	   are taken from applet.glade. */
@@ -316,89 +307,58 @@ nm_connection_editor_init (NMConnectionEditor *editor)
 	editor->window = glade_xml_get_widget (editor->xml, "nm-connection-editor");
 	editor->cancel_button = glade_xml_get_widget (editor->xml, "cancel_button");
 	editor->system_checkbutton = glade_xml_get_widget (editor->xml, "system_checkbutton");
+}
 
-	editor->system_action = polkit_action_new ();
-	polkit_action_set_action_id (editor->system_action, "org.freedesktop.network-manager-settings.system.modify");
-
-	editor->system_gnome_action = polkit_gnome_action_new ("system");
-
-	auth_label = _("Apply...");
-	auth_tooltip = _("Authenticate to save this connection for all users of this machine.");
-	label = _("Apply");
-	tooltip = _("Save any changes made to this connection.");
-	g_object_set (editor->system_gnome_action,
-	              "polkit-action", NULL,
-
-	              "self-blocked-visible",       TRUE,
-	              "self-blocked-sensitive",     FALSE,
-	              "self-blocked-short-label",   label,
-	              "self-blocked-label",         label,
-	              "self-blocked-tooltip",       tooltip,
-	              "self-blocked-icon-name",     GTK_STOCK_APPLY,
-
-	              "no-visible",       TRUE,
-	              "no-sensitive",     FALSE,
-	              "no-short-label",   label,
-	              "no-label",         label,
-	              "no-tooltip",       tooltip,
-	              "no-icon-name",     GTK_STOCK_APPLY,
-
-	              "auth-visible",     TRUE,
-	              "auth-sensitive",   TRUE,
-	              "auth-short-label", auth_label,
-	              "auth-label",       auth_label,
-	              "auth-tooltip",     auth_tooltip,
-	              "auth-icon-name",   GTK_STOCK_DIALOG_AUTHENTICATION,
-
-	              "yes-visible",      TRUE,
-	              "yes-sensitive",    TRUE,
-	              "yes-short-label",  label,
-	              "yes-label",        label,
-	              "yes-tooltip",      tooltip,
-	              "yes-icon-name",    GTK_STOCK_APPLY,
-
-	              "master-visible",   TRUE,
-	              "master-sensitive", TRUE,
-	              NULL);
-	g_signal_connect (editor->system_gnome_action, "polkit-result-changed",
-	                  G_CALLBACK (system_pk_result_changed_cb), editor);
-
-
-	editor->ok_button = polkit_gnome_action_create_button (editor->system_gnome_action);
-	hbox = glade_xml_get_widget (editor->xml, "action_area_hbox");
-	gtk_box_pack_end (GTK_BOX (hbox), editor->ok_button, TRUE, TRUE, 0);
+static void
+get_secrets_info_free (GetSecretsInfo *info)
+{
+	g_free (info->setting_name);
+	g_free (info);
 }
 
 static void
 dispose (GObject *object)
 {
 	NMConnectionEditor *editor = NM_CONNECTION_EDITOR (object);
+	GSList *iter;
+
+	editor->disposed = TRUE;
+
+	g_slist_foreach (editor->initializing_pages, (GFunc) g_object_unref, NULL);
+	g_slist_free (editor->initializing_pages);
+	editor->initializing_pages = NULL;
 
 	g_slist_foreach (editor->pages, (GFunc) g_object_unref, NULL);
 	g_slist_free (editor->pages);
 	editor->pages = NULL;
 
-	if (editor->connection) {
-                g_object_unref (editor->connection);
-                editor->connection = NULL;
-        }
-        if (editor->window) {
-                gtk_widget_destroy (editor->window);
-                editor->window = NULL;
-        }
-        if (editor->xml) {
-                g_object_unref (editor->xml);
-                editor->xml = NULL;
-        }
+	/* Mark any in-progress secrets call as canceled; it will clean up after itself. */
+	if (editor->secrets_call)
+		editor->secrets_call->canceled = TRUE;
 
-        if (editor->system_action) {
-                polkit_action_unref (editor->system_action);
-                editor->system_action = NULL;
-        }
-        if (editor->system_gnome_action) {
-                g_object_unref (editor->system_gnome_action);
-                editor->system_gnome_action = NULL;
-        }
+	/* Kill any pending secrets calls */
+	for (iter = editor->pending_secrets_calls; iter; iter = g_slist_next (iter)) {
+		get_secrets_info_free ((GetSecretsInfo *) iter->data);
+	}
+	g_slist_free (editor->pending_secrets_calls);
+	editor->pending_secrets_calls = NULL;
+
+	if (editor->connection) {
+		g_object_unref (editor->connection);
+		editor->connection = NULL;
+	}
+	if (editor->orig_connection) {
+		g_object_unref (editor->orig_connection);
+		editor->orig_connection = NULL;
+	}
+	if (editor->window) {
+		gtk_widget_destroy (editor->window);
+		editor->window = NULL;
+	}
+	if (editor->xml) {
+		g_object_unref (editor->xml);
+		editor->xml = NULL;
+	}
 
 	G_OBJECT_CLASS (nm_connection_editor_parent_class)->dispose (object);
 }
@@ -424,10 +384,12 @@ nm_connection_editor_class_init (NMConnectionEditorClass *klass)
 
 NMConnectionEditor *
 nm_connection_editor_new (NMConnection *connection,
-                          gboolean system_settings_can_modify,
+                          NMRemoteSettingsSystem *settings,
                           GError **error)
 {
 	NMConnectionEditor *editor;
+	GtkWidget *hbox;
+	gboolean sensitive = TRUE, use_polkit = FALSE;
 
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), NULL);
 
@@ -437,7 +399,40 @@ nm_connection_editor_new (NMConnection *connection,
 		return NULL;
 	}
 
-	editor->system_settings_can_modify = system_settings_can_modify;
+	g_object_get (settings,
+	              NM_SETTINGS_SYSTEM_INTERFACE_CAN_MODIFY,
+	              &editor->system_settings_can_modify,
+	              NULL);
+	g_signal_connect (settings,
+	                  "notify::" NM_SETTINGS_SYSTEM_INTERFACE_CAN_MODIFY,
+	                  G_CALLBACK (can_modify_changed_cb),
+	                  editor);
+
+	/* If this is a system connection that we can't ever modify,
+	 * set the editor's Apply button always insensitive.
+	 */
+	if (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_SYSTEM) {
+		sensitive = editor->system_settings_can_modify;
+		use_polkit = TRUE;
+	}
+
+	editor->ok_button = ce_polkit_button_new (_("Apply"),
+	                                          _("Save any changes made to this connection."),
+	                                          _("Apply..."),
+	                                          _("Authenticate to save this connection for all users of this machine."),
+	                                          GTK_STOCK_APPLY,
+	                                          settings,
+	                                          NM_SETTINGS_SYSTEM_PERMISSION_CONNECTION_MODIFY);
+	ce_polkit_button_set_use_polkit (CE_POLKIT_BUTTON (editor->ok_button), use_polkit);
+
+	g_signal_connect (editor->ok_button, "actionable",
+	                  G_CALLBACK (ok_button_actionable_cb), editor);
+	g_signal_connect (editor->ok_button, "authorized",
+	                  G_CALLBACK (ok_button_actionable_cb), editor);
+	hbox = glade_xml_get_widget (editor->xml, "action_area_hbox");
+	gtk_box_pack_end (GTK_BOX (hbox), editor->ok_button, TRUE, TRUE, 0);
+	gtk_widget_show_all (editor->ok_button);
+
 	if (!nm_connection_editor_set_connection (editor, connection, error)) {
 		g_object_unref (editor);
 		return NULL;
@@ -451,7 +446,27 @@ nm_connection_editor_get_connection (NMConnectionEditor *editor)
 {
 	g_return_val_if_fail (NM_IS_CONNECTION_EDITOR (editor), NULL);
 
-	return editor->connection;
+	return editor->orig_connection;
+}
+
+gboolean
+nm_connection_editor_update_connection (NMConnectionEditor *editor, GError **error)
+{
+	GHashTable *settings;
+
+	g_return_val_if_fail (NM_IS_CONNECTION_EDITOR (editor), FALSE);
+
+	if (!nm_connection_verify (editor->connection, error))
+		return FALSE;
+
+	/* Copy the modified connection to the original connection */
+	settings = nm_connection_to_hash (editor->connection);
+	nm_connection_replace_settings (editor->orig_connection, settings, NULL);
+	g_hash_table_destroy (settings);
+
+	nm_connection_set_scope (editor->orig_connection,
+	                         nm_connection_get_scope (editor->connection));
+	return TRUE;
 }
 
 static void
@@ -478,14 +493,12 @@ populate_connection_ui (NMConnectionEditor *editor)
 	g_signal_connect_swapped (name, "changed", G_CALLBACK (connection_editor_validate), editor);
 	g_signal_connect_swapped (autoconnect, "toggled", G_CALLBACK (connection_editor_validate), editor);
 
-	if (!editor->system_settings_can_modify)
-		gtk_widget_set_sensitive (editor->system_checkbutton, FALSE);
 	g_signal_connect (editor->system_checkbutton, "toggled", G_CALLBACK (system_checkbutton_toggled_cb), editor);
 
 	if (editor->orig_scope == NM_CONNECTION_SCOPE_SYSTEM)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (editor->system_checkbutton), TRUE);
 
-	update_sensitivity (editor, POLKIT_RESULT_UNKNOWN);
+	connection_editor_validate (editor);
 }
 
 static void
@@ -506,20 +519,13 @@ idle_validate (gpointer user_data)
 static void
 recheck_initialization (NMConnectionEditor *editor)
 {
-	GSList *iter;
-
-	/* Check if all pages are initialized; if not, desensitize the editor */
-	for (iter = editor->pages; iter; iter = g_slist_next (iter)) {
-		if (!ce_page_get_initialized (CE_PAGE (iter->data))) {
-			set_editor_sensitivity (editor, FALSE);
-			return;
-		}
-	}
+	if (!editor_is_initialized (editor))
+		return;
 
 	populate_connection_ui (editor);
-	update_sensitivity (editor, POLKIT_RESULT_UNKNOWN);
 
-	editor->initialized = TRUE;
+	/* When everything is initialized, re-present the window to ensure it's on top */
+	nm_connection_editor_present (editor);
 
 	/* Validate the connection from an idle handler to ensure that stuff like
 	 * GtkFileChoosers have had a chance to asynchronously find their files.
@@ -527,26 +533,13 @@ recheck_initialization (NMConnectionEditor *editor)
 	g_idle_add (idle_validate, editor);
 }
 
-static void
-really_add_page (NMConnectionEditor *editor, CEPage *page)
-{
-	GtkWidget *widget;
-	GtkWidget *notebook;
-	GtkWidget *label;
-
-	notebook = glade_xml_get_widget (editor->xml, "notebook");
-	label = gtk_label_new (ce_page_get_title (page));
-	widget = ce_page_get_page (page);
-	if (widget) {
-		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), widget, label);
-		g_object_set_data (G_OBJECT (page), "widget-added", GUINT_TO_POINTER (1));
-	}
-}
-
 static void
 page_initialized (CEPage *page, gpointer unused, GError *error, gpointer user_data)
 {
 	NMConnectionEditor *editor = NM_CONNECTION_EDITOR (user_data);
+	GtkWidget *widget;
+	GtkWidget *notebook;
+	GtkWidget *label;
 
 	if (error) {
 		gtk_widget_hide (editor->window);
@@ -554,19 +547,131 @@ page_initialized (CEPage *page, gpointer unused, GError *error, gpointer user_da
 		return;
 	}
 
-	/* If the page didn't get added the first time (because it didn't have secrets
-	 * or the widget couldn't be built until the secrets were requested) then do
-	 * that here.  This mainly happens for VPN system connections where the 
-	 * widget is constructed after the secrets call returns, which could be
-	 * a long time after ce_page_vpn_new() is called.
-	 */
+	/* Add the page to the UI */
+	notebook = glade_xml_get_widget (editor->xml, "notebook");
+	label = gtk_label_new (ce_page_get_title (page));
+	widget = ce_page_get_page (page);
+	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), widget, label);
 
-	if (!g_object_get_data (G_OBJECT (page), "widget-added"))
-		really_add_page (editor, page);
+	/* Move the page from the initializing list to the main page list */
+	editor->initializing_pages = g_slist_remove (editor->initializing_pages, page);
+	editor->pages = g_slist_append (editor->pages, page);
 
 	recheck_initialization (editor);
 }
 
+static void request_secrets (GetSecretsInfo *info);
+
+static void
+get_secrets_cb (NMSettingsConnectionInterface *connection,
+                GHashTable *secrets,
+                GError *error,
+                gpointer user_data)
+{
+	GetSecretsInfo *info = user_data;
+	NMConnectionEditor *self;
+
+	if (info->canceled) {
+		get_secrets_info_free (info);
+		return;
+	}
+
+	self = info->self;
+
+	/* Complete this secrets request; completion can actually dispose of the
+	 * dialog if there was an error.
+	 */
+	self->secrets_call = NULL;
+	ce_page_complete_init (info->page, info->setting_name, secrets, error);
+	get_secrets_info_free (info);
+
+	/* Kick off the next secrets request if there is one queued; if the dialog
+	 * was disposed of by the completion above we don't need to do anything.
+	 */
+	if (!self->disposed && self->pending_secrets_calls) {
+		self->secrets_call = g_slist_nth_data (self->pending_secrets_calls, 0);
+		self->pending_secrets_calls = g_slist_remove (self->pending_secrets_calls, self->secrets_call);
+
+		request_secrets (self->secrets_call);
+	}
+}
+
+static void
+request_secrets (GetSecretsInfo *info)
+{
+	NMSettingsConnectionInterface *connection;
+	gboolean success;
+	GError *error;
+
+	g_return_if_fail (info != NULL);
+
+	connection = NM_SETTINGS_CONNECTION_INTERFACE (info->self->orig_connection);
+	success = nm_settings_connection_interface_get_secrets (connection,
+	                                                        info->setting_name,
+	                                                        NULL,
+	                                                        FALSE,
+	                                                        get_secrets_cb,
+	                                                        info);
+	if (!success) {
+		error = g_error_new_literal (0, 0, _("Failed to update connection secrets due to an unknown error."));
+		get_secrets_cb (connection, NULL, error, info);
+		g_error_free (error);
+	}
+}
+
+static void
+get_secrets_for_page (NMConnectionEditor *self,
+                      CEPage *page,
+                      const char *setting_name)
+{
+	GetSecretsInfo *info;
+
+	if (!setting_name) {
+		/* page doesn't need any secrets */
+		ce_page_complete_init (page, NULL, NULL, NULL);
+		return;
+	}
+
+	/* Try to get secrets from ->orig_connection, because it's the one that
+	 * implements NMSettingsConnectionInterface and can respond to requests for
+	 * its secrets.  ->connection is a plain NMConnection copy of ->orig_connection
+	 * which is the connection that's actually changed when the user clicks stuff.
+	 * When creating importing or creating new connections though, ->orig_connection
+	 * is an NMConnection because it hasn't been exported over D-Bus yet, so we
+	 * can't ask it for secrets, because it doesn't implement NMSettingsConnectionInterface.
+	 */
+	if (!NM_IS_SETTINGS_CONNECTION_INTERFACE (self->orig_connection)) {
+		ce_page_complete_init (page, setting_name, NULL, NULL);
+		return;
+	}
+
+	info = g_malloc0 (sizeof (GetSecretsInfo));
+	info->self = self;
+	info->page = page;
+	info->setting_name = g_strdup (setting_name);
+
+	/* PolicyKit doesn't queue up authorization requests internally.  Instead,
+	 * if there's a pending authorization request, subsequent requests for that
+	 * same authorization will return NotAuthorized+Challenge.  That's pretty
+	 * inconvenient and it would be a lot nicer if PK just queued up subsequent
+	 * authorization requests and executed them when the first one was finished.
+	 * But it since it doesn't do that, we have to serialize the authorization
+	 * requests ourselves to get the right authorization result.
+	 */
+	/* NOTE: PolicyKit-gnome 0.95 now serializes auth requests as of this commit:
+	 * http://git.gnome.org/cgit/PolicyKit-gnome/commit/?id=f32cb7faa7197b9db55b569677732742c3c7fdc1
+	 */
+
+	/* If there's already an in-progress call, queue up the new one */
+	if (self->secrets_call)
+		self->pending_secrets_calls = g_slist_append (self->pending_secrets_calls, info);
+	else {
+		/* Request secrets for this page */
+		self->secrets_call = info;
+		request_secrets (info);
+	}
+}
+
 static gboolean
 add_page (NMConnectionEditor *editor,
           CEPageNewFunc func,
@@ -574,28 +679,31 @@ add_page (NMConnectionEditor *editor,
           GError **error)
 {
 	CEPage *page;
+	const char *secrets_setting_name = NULL;
 
 	g_return_val_if_fail (editor != NULL, FALSE);
 	g_return_val_if_fail (func != NULL, FALSE);
 	g_return_val_if_fail (connection != NULL, FALSE);
 
-	page = (*func) (connection, GTK_WINDOW (editor->window), error);
+	page = (*func) (connection, GTK_WINDOW (editor->window), &secrets_setting_name, error);
 	if (!page)
 		return FALSE;
 
-	really_add_page (editor, page);
-
-	editor->pages = g_slist_append (editor->pages, page);
-
+	editor->initializing_pages = g_slist_prepend (editor->initializing_pages, page);
 	g_signal_connect (page, "changed", G_CALLBACK (page_changed), editor);
 	g_signal_connect (page, "initialized", G_CALLBACK (page_initialized), editor);
 
+	/* Request any secrets the page might require; or if it doesn't want any,
+	 * let the page initialize.
+	 */
+	get_secrets_for_page (editor, page, secrets_setting_name);
+
 	return TRUE;
 }
 
 static gboolean
 nm_connection_editor_set_connection (NMConnectionEditor *editor,
-                                     NMConnection *connection,
+                                     NMConnection *orig_connection,
                                      GError **error)
 {
 	NMSettingConnection *s_con;
@@ -603,14 +711,16 @@ nm_connection_editor_set_connection (NMConnectionEditor *editor,
 	gboolean success = FALSE;
 
 	g_return_val_if_fail (NM_IS_CONNECTION_EDITOR (editor), FALSE);
-	g_return_val_if_fail (NM_IS_CONNECTION (connection), FALSE);
+	g_return_val_if_fail (NM_IS_CONNECTION (orig_connection), FALSE);
 
 	/* clean previous connection */
 	if (editor->connection)
 		g_object_unref (editor->connection);
 
-	editor->connection = g_object_ref (connection);
-	editor->orig_scope = nm_connection_get_scope (connection);
+	editor->connection = nm_connection_duplicate (orig_connection);
+
+	editor->orig_connection = g_object_ref (orig_connection);
+	editor->orig_scope = nm_connection_get_scope (editor->connection);
 	nm_connection_editor_update_title (editor);
 
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (editor->connection, NM_TYPE_SETTING_CONNECTION));
@@ -618,40 +728,44 @@ nm_connection_editor_set_connection (NMConnectionEditor *editor,
 
 	connection_type = nm_setting_connection_get_connection_type (s_con);
 	if (!strcmp (connection_type, NM_SETTING_WIRED_SETTING_NAME)) {
-		if (!add_page (editor, ce_page_wired_new, connection, error))
+		if (!add_page (editor, ce_page_wired_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_wired_security_new, connection, error))
+		if (!add_page (editor, ce_page_wired_security_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_ip4_new, connection, error))
+		if (!add_page (editor, ce_page_ip4_new, editor->connection, error))
+			goto out;
+		if (!add_page (editor, ce_page_ip6_new, editor->connection, error))
 			goto out;
 	} else if (!strcmp (connection_type, NM_SETTING_WIRELESS_SETTING_NAME)) {
-		if (!add_page (editor, ce_page_wireless_new, connection, error))
+		if (!add_page (editor, ce_page_wireless_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_wireless_security_new, connection, error))
+		if (!add_page (editor, ce_page_wireless_security_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_ip4_new, connection, error))
+		if (!add_page (editor, ce_page_ip4_new, editor->connection, error))
+			goto out;
+		if (!add_page (editor, ce_page_ip6_new, editor->connection, error))
 			goto out;
 	} else if (!strcmp (connection_type, NM_SETTING_VPN_SETTING_NAME)) {
-		if (!add_page (editor, ce_page_vpn_new, connection, error))
+		if (!add_page (editor, ce_page_vpn_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_ip4_new, connection, error))
+		if (!add_page (editor, ce_page_ip4_new, editor->connection, error))
 			goto out;
 	} else if (!strcmp (connection_type, NM_SETTING_PPPOE_SETTING_NAME)) {
-		if (!add_page (editor, ce_page_dsl_new, connection, error))
+		if (!add_page (editor, ce_page_dsl_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_wired_new, connection, error))
+		if (!add_page (editor, ce_page_wired_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_ppp_new, connection, error))
+		if (!add_page (editor, ce_page_ppp_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_ip4_new, connection, error))
+		if (!add_page (editor, ce_page_ip4_new, editor->connection, error))
 			goto out;
 	} else if (!strcmp (connection_type, NM_SETTING_GSM_SETTING_NAME) || 
 	           !strcmp (connection_type, NM_SETTING_CDMA_SETTING_NAME)) {
-		if (!add_page (editor, ce_page_mobile_new, connection, error))
+		if (!add_page (editor, ce_page_mobile_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_ppp_new, connection, error))
+		if (!add_page (editor, ce_page_ppp_new, editor->connection, error))
 			goto out;
-		if (!add_page (editor, ce_page_ip4_new, connection, error))
+		if (!add_page (editor, ce_page_ip4_new, editor->connection, error))
 			goto out;
 	} else {
 		g_warning ("Unhandled setting type '%s'", connection_type);
@@ -678,7 +792,6 @@ cancel_button_clicked_cb (GtkWidget *widget, gpointer user_data)
 {
 	NMConnectionEditor *self = NM_CONNECTION_EDITOR (user_data);
 
-	gtk_widget_hide (widget);
 	g_signal_emit (self, editor_signals[EDITOR_DONE], 0, GTK_RESPONSE_CANCEL, NULL);
 }
 
@@ -693,7 +806,6 @@ ok_button_clicked_cb (GtkWidget *widget, gpointer user_data)
 {
 	NMConnectionEditor *self = NM_CONNECTION_EDITOR (user_data);
 
-	gtk_widget_hide (widget);
 	g_signal_emit (self, editor_signals[EDITOR_DONE], 0, GTK_RESPONSE_OK, NULL);
 }
 
@@ -739,3 +851,22 @@ nm_connection_editor_get_window (NMConnectionEditor *editor)
 	return GTK_WINDOW (editor->window);
 }
 
+gboolean
+nm_connection_editor_get_busy (NMConnectionEditor *editor)
+{
+	g_return_val_if_fail (NM_IS_CONNECTION_EDITOR (editor), FALSE);
+
+	return editor->busy;
+}
+
+void
+nm_connection_editor_set_busy (NMConnectionEditor *editor, gboolean busy)
+{
+	g_return_if_fail (NM_IS_CONNECTION_EDITOR (editor));
+
+	if (busy != editor->busy) {
+		editor->busy = busy;
+		gtk_widget_set_sensitive (editor->window, !busy);
+	}
+}
+
