@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2004-2005 Red Hat, Inc.
+ * (C) Copyright 2004-2009 Red Hat, Inc.
  */
 
 #ifndef NM_CONNECTION_LIST_H
@@ -28,7 +28,7 @@
 #include <gconf/gconf-client.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
-#include <nm-dbus-settings-system.h>
+#include <nm-remote-settings-system.h>
 #include "nma-gconf-settings.h"
 
 #define NM_TYPE_CONNECTION_LIST    (nm_connection_list_get_type ())
@@ -40,11 +40,11 @@ typedef struct {
 
 	/* private data */
 	GHashTable *editors;
-	GHashTable *treeviews;
+	GSList *treeviews;
 
 	GConfClient *client;
 	NMAGConfSettings *gconf_settings;
-	NMDBusSettingsSystem *system_settings;
+	NMRemoteSettingsSystem *system_settings;
 
 	GladeXML *gui;
 	GtkWidget *dialog;
@@ -65,11 +65,11 @@ typedef struct {
 } NMConnectionListClass;
 
 GType             nm_connection_list_get_type (void);
-NMConnectionList *nm_connection_list_new (const char *def_type);
+NMConnectionList *nm_connection_list_new (GType def_type);
 
 void              nm_connection_list_run (NMConnectionList *list);
 
-void              nm_connection_list_set_type (NMConnectionList *list, const char *type);
+void              nm_connection_list_set_type (NMConnectionList *list, GType ctype);
 
 void              nm_connection_list_present (NMConnectionList *list);
 
