@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 Red Hat, Inc.
+ * (C) Copyright 2008 - 2010 Red Hat, Inc.
  */
 
 #include <string.h>
@@ -235,7 +235,7 @@ finish_setup (CEPageWirelessSecurity *self, gpointer unused, GError *error, gpoi
 	if (nm_utils_security_valid (NMU_SEC_NONE, dev_caps, FALSE, is_adhoc, 0, 0, 0)) {
 		gtk_list_store_append (sec_model, &iter);
 		gtk_list_store_set (sec_model, &iter,
-		                    S_NAME_COLUMN, _("None"),
+		                    S_NAME_COLUMN, C_("No wifi security used", "None"),
 		                    -1);
 		if (default_type == NMU_SEC_NONE)
 			active = item;
@@ -291,7 +291,7 @@ finish_setup (CEPageWirelessSecurity *self, gpointer unused, GError *error, gpoi
 	if (nm_utils_security_valid (NMU_SEC_DYNAMIC_WEP, dev_caps, FALSE, is_adhoc, 0, 0, 0)) {
 		WirelessSecurityDynamicWEP *ws_dynamic_wep;
 
-		ws_dynamic_wep = ws_dynamic_wep_new (glade_file, connection);
+		ws_dynamic_wep = ws_dynamic_wep_new (glade_file, connection, TRUE);
 		if (ws_dynamic_wep) {
 			add_security_item (self, WIRELESS_SECURITY (ws_dynamic_wep), sec_model,
 			                   &iter, _("Dynamic WEP (802.1x)"));
@@ -319,7 +319,7 @@ finish_setup (CEPageWirelessSecurity *self, gpointer unused, GError *error, gpoi
 	    || nm_utils_security_valid (NMU_SEC_WPA2_ENTERPRISE, dev_caps, FALSE, is_adhoc, 0, 0, 0)) {
 		WirelessSecurityWPAEAP *ws_wpa_eap;
 
-		ws_wpa_eap = ws_wpa_eap_new (glade_file, connection);
+		ws_wpa_eap = ws_wpa_eap_new (glade_file, connection, TRUE);
 		if (ws_wpa_eap) {
 			add_security_item (self, WIRELESS_SECURITY (ws_wpa_eap), sec_model,
 			                   &iter, _("WPA & WPA2 Enterprise"));
@@ -342,13 +342,15 @@ finish_setup (CEPageWirelessSecurity *self, gpointer unused, GError *error, gpoi
 }
 
 CEPage *
-ce_page_wireless_security_new (NMConnection *connection, GtkWindow *parent_window, GError **error)
+ce_page_wireless_security_new (NMConnection *connection,
+                               GtkWindow *parent_window,
+                               const char **out_secrets_setting_name,
+                               GError **error)
 {
 	CEPageWirelessSecurity *self;
 	CEPage *parent;
 	NMSettingWireless *s_wireless;
 	NMSettingWirelessSecurity *s_wsec = NULL;
-	const char *setting_name = NULL;
 	NMUtilsSecurityType default_type = NMU_SEC_NONE;
 	const char *security;
 
@@ -398,21 +400,17 @@ ce_page_wireless_security_new (NMConnection *connection, GtkWindow *parent_windo
 	    || default_type == NMU_SEC_LEAP
 	    || default_type == NMU_SEC_WPA_PSK
 	    || default_type == NMU_SEC_WPA2_PSK) {
-		setting_name = NM_SETTING_WIRELESS_SECURITY_SETTING_NAME;
+		*out_secrets_setting_name = NM_SETTING_WIRELESS_SECURITY_SETTING_NAME;
 	}
 
 	/* Or if it is 802.1x enabled */
 	if (   default_type == NMU_SEC_DYNAMIC_WEP
 	    || default_type == NMU_SEC_WPA_ENTERPRISE
 	    || default_type == NMU_SEC_WPA2_ENTERPRISE) {
-		setting_name = NM_SETTING_802_1X_SETTING_NAME;
+		*out_secrets_setting_name = NM_SETTING_802_1X_SETTING_NAME;
 	}
 
 	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
-	if (!ce_page_initialize (parent, setting_name, error)) {
-		g_object_unref (self);
-		return NULL;
-	}
 
 	return CE_PAGE (self);
 }
@@ -468,7 +466,6 @@ validate (CEPage *page, NMConnection *connection, GError **error)
 		g_object_set (s_wireless, NM_SETTING_WIRELESS_SEC, NULL, NULL);
 		nm_connection_remove_setting (connection, NM_TYPE_SETTING_WIRELESS_SECURITY);
 		nm_connection_remove_setting (connection, NM_TYPE_SETTING_802_1X);
-		nm_gconf_clear_private_connection_values (connection);
 		valid = TRUE;
 	}
 
