@@ -211,7 +211,6 @@ import_vpn_from_file_cb (GtkWidget *dialog, gint response, gpointer user_data)
 	import_info.connection = NULL;
 	import_info.error = NULL;
 	g_hash_table_foreach (plugins, try_import, (gpointer) &import_info);
-	g_free (import_info.filename);
 
 	connection = import_info.connection;
 	if (connection) {
@@ -239,6 +238,7 @@ import_vpn_from_file_cb (GtkWidget *dialog, gint response, gpointer user_data)
 
 	if (import_info.error)
 		g_error_free (import_info.error);
+	g_free (filename);
 
 out:
 	gtk_widget_hide (dialog);
@@ -471,7 +471,7 @@ error:
 }
 
 char *
-vpn_ask_connection_type (void)
+vpn_ask_connection_type (GtkWindow *parent)
 {
 	GladeXML *xml;
 	GtkWidget *dialog, *combo, *widget;
@@ -521,6 +521,7 @@ vpn_ask_connection_type (void)
 	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), model);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
 	gtk_widget_show_all (dialog);
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (response != GTK_RESPONSE_OK)
