@@ -426,6 +426,20 @@ clear_keyring_items (NMAGConfConnection *self)
 	}
 }
 
+void
+nma_gconf_connection_update (NMAGConfConnection *self,
+                             gboolean ignore_secrets)
+{
+	NMAGConfConnectionPrivate *priv = NMA_GCONF_CONNECTION_GET_PRIVATE (self);
+
+	nm_gconf_write_connection (NM_CONNECTION (self),
+	                           priv->client,
+	                           priv->dir,
+	                           ignore_secrets);
+	gconf_client_notify (priv->client, priv->dir);
+	gconf_client_suggest_sync (priv->client, NULL);
+}
+
 /******************************************************/
 
 static gboolean
@@ -433,13 +447,10 @@ update (NMSettingsConnectionInterface *connection,
 	    NMSettingsConnectionInterfaceUpdateFunc callback,
 	    gpointer user_data)
 {
-	NMAGConfConnectionPrivate *priv = NMA_GCONF_CONNECTION_GET_PRIVATE (connection);
-
-	nm_gconf_write_connection (NM_CONNECTION (connection),
-	                           priv->client,
-	                           priv->dir);
-	gconf_client_notify (priv->client, priv->dir);
-	gconf_client_suggest_sync (priv->client, NULL);
+	/* Always update secrets since it's assumed that secrets are included in
+	 * the new connection data.
+	 */
+	nma_gconf_connection_update (NMA_GCONF_CONNECTION (connection), FALSE);
 
 	return parent_settings_connection_iface->update (connection, callback, user_data);
 }
@@ -467,6 +478,43 @@ do_delete (NMSettingsConnectionInterface *connection,
 	return parent_settings_connection_iface->delete (connection, callback, user_data);
 }
 
+static gboolean
+is_otp_always_ask (NMConnection *connection)
+{
+	NMSetting8021x *s_8021x;
+	NMSettingConnection *s_con;
+	const char *uuid, *eap_method, *phase2;
+
+	s_8021x = (NMSetting8021x *) nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X);
+	if (s_8021x) {
+		gboolean can_always_ask = FALSE;
+
+		/* Check if PEAP or TTLS is used */
+		eap_method = nm_setting_802_1x_get_eap_method (s_8021x, 0);
+		s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+		if (!strcmp (eap_method, "peap"))
+			can_always_ask = TRUE;
+		else if (!strcmp (eap_method, "ttls")) {
+			/* Now make sure the phase2 method isn't TLS */
+			phase2 = nm_setting_802_1x_get_phase2_auth (s_8021x);
+			if (phase2 && strcmp (phase2, "tls"))
+				can_always_ask = TRUE;
+			else {
+				phase2 = nm_setting_802_1x_get_phase2_autheap (s_8021x);
+				if (phase2 && strcmp (phase2, "tls"))
+					can_always_ask = TRUE;
+			}
+		}
+
+		if (can_always_ask) {
+			uuid = nm_setting_connection_get_uuid (s_con);
+			if (nm_gconf_get_8021x_password_always_ask (uuid))
+				return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 static gboolean
 internal_get_secrets (NMSettingsConnectionInterface *connection,
                       const char *setting_name,
@@ -528,6 +576,10 @@ internal_get_secrets (NMSettingsConnectionInterface *connection,
 			goto get_secrets;
 		}
 
+		/* OTP connections marked as always ask */
+		if (is_otp_always_ask (NM_CONNECTION (self)))
+			goto get_secrets;
+
 		secrets = nma_gconf_connection_get_keyring_items (self, setting_name, error);
 		if (!secrets) {
 			if (error && *error)
