@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2008 Red Hat, Inc.
+ * (C) Copyright 2007 - 2010 Red Hat, Inc.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,6 +51,12 @@ G_DEFINE_TYPE (NMAWirelessDialog, nma_wireless_dialog, GTK_TYPE_DIALOG)
                                             NMA_TYPE_WIRELESS_DIALOG, \
                                             NMAWirelessDialogPrivate))
 
+typedef struct {
+	NMAWirelessDialog *self;
+	NMConnection *connection;
+	gboolean canceled;
+} GetSecretsInfo;
+
 typedef struct {
 	NMApplet *applet;
 
@@ -72,6 +78,10 @@ typedef struct {
 
 	gboolean auth_only;
 
+	guint revalidate_id;
+
+	GetSecretsInfo *secrets_info;
+
 	gboolean disposed;
 } NMAWirelessDialogPrivate;
 
@@ -208,7 +218,7 @@ security_combo_changed (GtkWidget *combo,
 	/* Set focus to the security method's default widget, but only if the
 	 * network name entry should not be focused.
 	 */
-	if (!priv->network_name_focus) {
+	if (!priv->network_name_focus && sec->default_field) {
 		def_widget = glade_xml_get_widget (sec->xml, sec->default_field);
 		if (def_widget)
 			gtk_widget_grab_focus (def_widget);
@@ -287,10 +297,17 @@ stuff_changed_cb (WirelessSecurity *sec, gpointer user_data)
 			g_byte_array_free (ssid, TRUE);
 	}
 
-	/* Assume system connections are valid so that we don't have to get secrets for them. */
+	/* Assume system connections are valid so that we don't have to get secrets for them */
 	if (is_system_connection (self))
 		valid = TRUE;
 
+	/* But if there's an in-progress secrets call (which might require authorization)
+	 * then we don't want to enable the OK button because we don't have all the
+	 * connection details yet.
+	 */
+	if (priv->secrets_info)
+		valid = FALSE;
+
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, valid);
 }
 
@@ -326,10 +343,17 @@ ssid_entry_changed (GtkWidget *entry, gpointer user_data)
 	}
 
 out:
-	/* Assume system connections are valid so that we don't have to get secrets for them. */
+	/* Assume system connections are valid so that we don't have to get secrets for them */
 	if (is_system_connection (self))
 		valid = TRUE;
 
+	/* But if there's an in-progress secrets call (which might require authorization)
+	 * then we don't want to enable the OK button because we don't have all the
+	 * connection details yet.
+	 */
+	if (priv->secrets_info)
+		valid = FALSE;
+
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, valid);
 }
 
@@ -382,29 +406,11 @@ connection_combo_changed (GtkWidget *combo,
 	gtk_widget_set_sensitive (glade_xml_get_widget (priv->xml, "security_vbox"), is_new);
 }
 
-static void
-exported_connection_to_connection (gpointer data, gpointer user_data)
-{
-	GSList **list = (GSList **) user_data;
-
-	*list = g_slist_prepend (*list, nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (data)));
-}
-
 static GSList *
 get_all_connections (NMApplet *applet)
 {
-	GSList *list;
-	GSList *connections = NULL;
-
-	list = nm_settings_list_connections (NM_SETTINGS (applet->dbus_settings));
-	g_slist_foreach (list, exported_connection_to_connection, &connections);
-	g_slist_free (list);
-
-	list = nm_settings_list_connections (NM_SETTINGS (applet->gconf_settings));
-	g_slist_foreach (list, exported_connection_to_connection, &connections);
-	g_slist_free (list);
-
-	return connections;
+	return g_slist_concat (nm_settings_interface_list_connections (NM_SETTINGS_INTERFACE (applet->system_settings)),
+	                       nm_settings_interface_list_connections (NM_SETTINGS_INTERFACE (applet->gconf_settings)));
 }
 
 static gboolean
@@ -757,6 +763,88 @@ add_security_item (NMAWirelessDialog *self,
 	wireless_security_unref (sec);
 }
 
+static void
+get_secrets_cb (NMSettingsConnectionInterface *connection,
+                GHashTable *secrets,
+                GError *error,
+                gpointer user_data)
+{
+	GetSecretsInfo *info = user_data;
+	NMAWirelessDialogPrivate *priv;
+	GHashTableIter hash_iter;
+	gpointer key, value;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	if (info->canceled)
+		goto out;
+
+	priv = NMA_WIRELESS_DIALOG_GET_PRIVATE (info->self);
+	if (priv->secrets_info == info) {
+		priv->secrets_info = NULL;
+
+		/* Buttons should only be re-enabled if this secrets response is the
+		 * in-progress one.
+		 */
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (info->self), GTK_RESPONSE_CANCEL, TRUE);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (info->self), GTK_RESPONSE_OK, TRUE);
+	}
+
+	if (error) {
+		g_warning ("%s: error getting connection secrets: (%d) %s",
+		           __func__,
+		           error ? error->code : -1,
+		           error && error->message ? error->message : "(unknown)");
+		goto out;
+	}
+
+	/* User might have changed the connection while the secrets call was in
+	 * progress, so don't try to update the wrong connection with the secrets
+	 * we just received.
+	 */
+	if (   (priv->connection != info->connection)
+	    || !secrets
+	    || !g_hash_table_size (secrets))
+		goto out;
+
+	/* Try to update the connection's secrets; log errors but we don't care */
+	g_hash_table_iter_init (&hash_iter, secrets);
+	while (g_hash_table_iter_next (&hash_iter, &key, &value)) {
+		GError *update_error = NULL;
+		const char *setting_name = key;
+		GHashTable *setting_hash = value;
+
+		if (!nm_connection_update_secrets (priv->connection,
+		                                   setting_name,
+		                                   setting_hash,
+		                                   &update_error)) {
+			g_warning ("%s: error updating connection secrets: (%d) %s",
+			           __func__,
+			           update_error ? update_error->code : -1,
+			           update_error && update_error->message ? update_error->message : "(unknown)");
+			g_clear_error (&update_error);
+		}
+	}
+
+	/* Update each security method's UI elements with the new secrets */
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->sec_combo));
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		do {
+			WirelessSecurity *sec = NULL;
+
+			gtk_tree_model_get (model, &iter, S_SEC_COLUMN, &sec, -1);
+			if (sec) {
+				wireless_security_update_secrets (sec, priv->connection);
+				wireless_security_unref (sec);
+			}
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+
+out:
+	g_object_unref (info->connection);
+	g_free (info);
+}
+
 static gboolean
 security_combo_init (NMAWirelessDialog *self, gboolean auth_only)
 {
@@ -828,7 +916,7 @@ security_combo_init (NMAWirelessDialog *self, gboolean auth_only)
 	if (nm_utils_security_valid (NMU_SEC_NONE, dev_caps, !!priv->ap, is_adhoc, ap_flags, ap_wpa, ap_rsn)) {
 		gtk_list_store_append (sec_model, &iter);
 		gtk_list_store_set (sec_model, &iter,
-		                    S_NAME_COLUMN, _("None"),
+		                    S_NAME_COLUMN, C_("No wifi security used", "None"),
 		                    -1);
 		if (default_type == NMU_SEC_NONE)
 			active = item;
@@ -881,7 +969,7 @@ security_combo_init (NMAWirelessDialog *self, gboolean auth_only)
 	if (nm_utils_security_valid (NMU_SEC_DYNAMIC_WEP, dev_caps, !!priv->ap, is_adhoc, ap_flags, ap_wpa, ap_rsn)) {
 		WirelessSecurityDynamicWEP *ws_dynamic_wep;
 
-		ws_dynamic_wep = ws_dynamic_wep_new (priv->glade_file, priv->connection);
+		ws_dynamic_wep = ws_dynamic_wep_new (priv->glade_file, priv->connection, FALSE);
 		if (ws_dynamic_wep) {
 			add_security_item (self, WIRELESS_SECURITY (ws_dynamic_wep), sec_model,
 			                   &iter, _("Dynamic WEP (802.1x)"));
@@ -909,7 +997,7 @@ security_combo_init (NMAWirelessDialog *self, gboolean auth_only)
 	    || nm_utils_security_valid (NMU_SEC_WPA2_ENTERPRISE, dev_caps, !!priv->ap, is_adhoc, ap_flags, ap_wpa, ap_rsn)) {
 		WirelessSecurityWPAEAP *ws_wpa_eap;
 
-		ws_wpa_eap = ws_wpa_eap_new (priv->glade_file, priv->connection);
+		ws_wpa_eap = ws_wpa_eap_new (priv->glade_file, priv->connection, FALSE);
 		if (ws_wpa_eap) {
 			add_security_item (self, WIRELESS_SECURITY (ws_wpa_eap), sec_model,
 			                   &iter, _("WPA & WPA2 Enterprise"));
@@ -922,6 +1010,35 @@ security_combo_init (NMAWirelessDialog *self, gboolean auth_only)
 	gtk_combo_box_set_model (GTK_COMBO_BOX (priv->sec_combo), GTK_TREE_MODEL (sec_model));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (priv->sec_combo), active < 0 ? 0 : (guint32) active);
 	g_object_unref (G_OBJECT (sec_model));
+
+	/* Request secrets for the connection if it needs any */
+	if (NM_IS_SETTINGS_CONNECTION_INTERFACE (priv->connection)) {
+		const char *setting_name;
+
+		setting_name = nm_connection_need_secrets (priv->connection, NULL);
+		if (setting_name) {
+			GetSecretsInfo *info;
+
+			/* Desensitize the dialog's buttons while we wait for the secrets
+			 * operation to complete.
+			 */
+			gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, FALSE);
+			gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_CANCEL, FALSE);
+
+			info = g_malloc0 (sizeof (GetSecretsInfo));
+			info->self = self;
+			info->connection = g_object_ref (priv->connection);
+			priv->secrets_info = info;
+
+			nm_settings_connection_interface_get_secrets (NM_SETTINGS_CONNECTION_INTERFACE (priv->connection),
+			                                              setting_name,
+			                                              NULL,
+			                                              FALSE,
+			                                              get_secrets_cb,
+			                                              info);
+		}
+	}
+
 	return TRUE;
 }
 
@@ -931,6 +1048,7 @@ revalidate (gpointer user_data)
 	NMAWirelessDialog *self = NMA_WIRELESS_DIALOG (user_data);
 	NMAWirelessDialogPrivate *priv = NMA_WIRELESS_DIALOG_GET_PRIVATE (self);
 
+	priv->revalidate_id = 0;
 	security_combo_changed (priv->sec_combo, self);
 	return FALSE;
 }
@@ -963,11 +1081,7 @@ internal_init (NMAWirelessDialog *self,
 	widget = glade_xml_get_widget (priv->xml, "image1");
 	gtk_image_set_from_icon_name (GTK_IMAGE (widget), icon_name, GTK_ICON_SIZE_DIALOG);
 
-#if GTK_CHECK_VERSION (2,14,0)
 	gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))), 2);
-#else
-	gtk_box_set_spacing (GTK_BOX (gtk_bin_get_child (GTK_BIN (self))), 2);
-#endif
 
 	widget = gtk_dialog_add_button (GTK_DIALOG (self), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	gtk_box_set_child_packing (GTK_BOX (GTK_DIALOG (self)->action_area), widget,
@@ -997,11 +1111,7 @@ internal_init (NMAWirelessDialog *self,
 		return FALSE;
 	}
 
-#if GTK_CHECK_VERSION (2,14,0)
 	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (self))), widget);
-#else
-	gtk_container_add (GTK_CONTAINER (gtk_bin_get_child (GTK_BIN (self))), widget);
-#endif
 
 	/* If given a valid connection, hide the SSID bits and connection combo */
 	if (specific_connection) {
@@ -1019,7 +1129,7 @@ internal_init (NMAWirelessDialog *self,
 		priv->network_name_focus = TRUE;
 	}
 
-	/* Assume system connections are valid so that we don't have to get secrets for them. */
+	/* Assume system connections are valid so that we don't have to get secrets for them */
 	if (is_system_connection (self))
 		gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_OK, TRUE);
 	else
@@ -1089,7 +1199,7 @@ internal_init (NMAWirelessDialog *self,
 	/* Re-validate from an idle handler so that widgets like file choosers
 	 * have had time to find their files.
 	 */
-	g_idle_add (revalidate, self);
+	priv->revalidate_id = g_idle_add (revalidate, self);
 
 	return TRUE;
 }
@@ -1305,6 +1415,9 @@ dispose (GObject *object)
 
 	priv->disposed = TRUE;
 
+	if (priv->secrets_info)
+		priv->secrets_info->canceled = TRUE;
+
 	g_free (priv->glade_file);
 
 	g_object_unref (priv->xml);
@@ -1324,6 +1437,9 @@ dispose (GObject *object)
 	if (priv->ap)
 		g_object_unref (priv->ap);
 
+	if (priv->revalidate_id)
+		g_source_remove (priv->revalidate_id);
+
 	G_OBJECT_CLASS (nma_wireless_dialog_parent_class)->dispose (object);
 }
 
